@@ -95,7 +95,7 @@ def run():
   global final, target, script_src, script_inline
 
   if DEBUG: logging.warning('invocation: ' + ' '.join(sys.argv) + (' + ' + EMCC_CFLAGS if EMCC_CFLAGS else '') + '  (in ' + os.getcwd() + ')')
-  if EMCC_CFLAGS: sys.argv.append(EMCC_CFLAGS)
+  if EMCC_CFLAGS: sys.argv.extend(shlex.split(EMCC_CFLAGS))
 
   if DEBUG and LEAVE_INPUTS_RAW: logging.warning('leaving inputs raw')
 
@@ -128,9 +128,9 @@ def run():
         break
 
   if len(sys.argv) == 1 or '--help' in sys.argv:
-    # Documentation for emcc and its options must be updated in: 
+    # Documentation for emcc and its options must be updated in:
     #    site/source/docs/tools_reference/emcc.rst
-    # A prebuilt local version of the documentation is available at: 
+    # A prebuilt local version of the documentation is available at:
     #    site/build/text/docs/tools_reference/emcc.txt
     #    (it is read from there and printed out when --help is invoked)
     # You can also build docs locally as HTML or other formats in site/
@@ -241,7 +241,7 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
         idx += 1
 
     if compiler == shared.EMCC: compiler = [shared.PYTHON, shared.EMCC]
-    else: compiler = [compiler] 
+    else: compiler = [compiler]
     cmd = compiler + list(filter_emscripten_options(sys.argv[1:]))
     if not use_js: cmd += shared.EMSDK_OPTS + ['-D__EMSCRIPTEN__', '-DEMSCRIPTEN']
     if use_js: cmd += ['-s', 'ERROR_ON_UNDEFINED_SYMBOLS=1'] # configure tests should fail when an undefined symbol exists
@@ -381,10 +381,16 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
   # Log out times for emcc stages
   class TimeLogger:
     last = time.time()
+
+    @staticmethod
+    def update():
+      TimeLogger.last = time.time()
+
   def log_time(name):
-    now = time.time()
-    logging.debug('emcc step "%s" took %.2f seconds', name, now - TimeLogger.last)
-    TimeLogger.last = now
+    if DEBUG:
+      now = time.time()
+      logging.debug('emcc step "%s" took %.2f seconds', name, now - TimeLogger.last)
+      TimeLogger.update()
 
   misc_temp_files = shared.configuration.get_temp_files()
 
@@ -476,6 +482,24 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
         raise Exception(err_msg)
       return level
 
+    def detect_fixed_language_mode(args):
+      check_next = False
+      for item in args:
+        if check_next:
+          if item in ("c++", "c"):
+            return True
+          else:
+            check_next = False
+        if item.startswith("-x"):
+          lmode = item[2:] if len(item) > 2 else None
+          if lmode in ("c++", "c"):
+            return True
+          else:
+            check_next = True
+            continue
+      return False
+
+    has_fixed_language_mode = detect_fixed_language_mode(newargs)
     should_exit = False
 
     for i in range(len(newargs)):
@@ -631,8 +655,10 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
         shared.check_sanity(force=True) # this is a good time for a sanity check
         should_exit = True
       elif newargs[i] == '--clear-ports':
-        logging.warning('clearing ports')
+        logging.warning('clearing ports and cache')
         system_libs.Ports.erase()
+        shared.Cache.erase()
+        shared.check_sanity(force=True) # this is a good time for a sanity check
         should_exit = True
       elif newargs[i] == '--show-ports':
         system_libs.show_ports()
@@ -741,8 +767,10 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     for i in range(len(newargs)):
       if newargs[i] == '-s':
         if is_minus_s_for_emcc(newargs, i):
-          settings_changes.append(newargs[i+1])
+          key = newargs[i+1]
+          settings_changes.append(key)
           newargs[i] = newargs[i+1] = ''
+          assert key != 'WASM_BACKEND', 'do not set -s WASM_BACKEND, instead set EMCC_WASM_BACKEND=1 in the environment'
     newargs = [arg for arg in newargs if arg is not '']
 
     # Find input files
@@ -812,8 +840,13 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
               logging.error(arg + ': Unknown format, not a static library!')
             exit(1)
         else:
-          logging.error(arg + ": Input file has an unknown suffix, don't know what to do with it!")
-          exit(1)
+          if has_fixed_language_mode:
+            newargs[i] = ''
+            input_files.append((i, arg))
+            has_source_inputs = True
+          else:
+            logging.error(arg + ": Input file has an unknown suffix, don't know what to do with it!")
+            exit(1)
       elif arg.startswith('-L'):
         lib_dirs.append(arg[2:])
         newargs[i] = ''
@@ -854,12 +887,14 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     js_target = unsuffixed(target) + '.js'
 
     asm_target = js_target[:-3] + '.asm.js' # might not be used, but if it is, this is the name
-    wasm_target = asm_target.replace('.asm.js', '.wast') # ditto, might not be used
-    if final_suffix == 'html' and not separate_asm and 'PRECISE_F32=2' in settings_changes or 'USE_PTHREADS=2' in settings_changes:
+    wasm_text_target = asm_target.replace('.asm.js', '.wast') # ditto, might not be used
+    wasm_binary_target = asm_target.replace('.asm.js', '.wasm') # ditto, might not be used
+
+    if final_suffix == 'html' and not separate_asm and ('PRECISE_F32=2' in settings_changes or 'USE_PTHREADS=2' in settings_changes):
       separate_asm = True
       logging.warning('forcing separate asm output (--separate-asm), because -s PRECISE_F32=2 or -s USE_PTHREADS=2 was passed.')
     if separate_asm:
-      shared.Settings.SEPARATE_ASM = asm_target
+      shared.Settings.SEPARATE_ASM = os.path.basename(asm_target)
 
     # Find library files
     for i, lib in libs:
@@ -939,6 +974,11 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     if shared.Settings.ASSERTIONS:
       shared.Settings.STACK_OVERFLOW_CHECK = 2
 
+    if shared.get_llvm_target() == shared.WASM_TARGET:
+      shared.Settings.WASM_BACKEND = 1
+
+    # Use settings
+
     try:
       assert shared.Settings.ASM_JS > 0, 'ASM_JS must be enabled in fastcomp'
       assert shared.Settings.SAFE_HEAP in [0, 1], 'safe heap must be 0 or 1 in fastcomp'
@@ -1102,10 +1142,29 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
         logging.error('-s MAIN_MODULE=1 is not supported with -s USE_PTHREADS=1!')
         exit(1)
 
+    if shared.Settings.WASM_BACKEND:
+      shared.Settings.BINARYEN = 1
+      # Static linking is tricky with LLVM, since e.g. memset might not be used from libc,
+      # but be used as an intrinsic, and codegen will generate a libc call from that intrinsic
+      # *after* static linking would have thought it is all in there. In asm.js this is not an
+      # issue as we do JS linking anyhow, and have asm.js-optimized versions of all the LLVM
+      # intrinsics. But for wasm, we need a better solution. For now, just pin stuff.
+      shared.Settings.EXPORTED_FUNCTIONS += ['_memcpy', '_memmove', '_memset']
+      # to bootstrap struct_info, we need binaryen
+      os.environ['EMCC_WASM_BACKEND_BINARYEN'] = '1'
+
     if shared.Settings.BINARYEN:
       debug_level = max(1, debug_level) # keep whitespace readable, for asm.js parser simplicity
       shared.Settings.GLOBAL_BASE = 1024 # leave some room for mapping global vars
       assert not shared.Settings.SPLIT_MEMORY, 'WebAssembly does not support split memory'
+      if not shared.Settings.BINARYEN_METHOD:
+        shared.Settings.BINARYEN_METHOD = 'native-wasm,interpret-binary'
+
+    if shared.Settings.CYBERDWARF:
+      newargs.append('-g')
+      shared.Settings.BUNDLED_CD_DEBUG_FILE = target + ".cd"
+      js_libraries.append(shared.path_from_root('src', 'library_cyberdwarf.js'))
+      js_libraries.append(shared.path_from_root('src', 'library_debugger_toolkit.js'))
 
     if tracing:
       if shared.Settings.ALLOW_MEMORY_GROWTH:
@@ -1183,19 +1242,22 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       logging.debug(('just preprocessor ' if '-E' in newargs else 'just dependencies: ') + ' '.join(cmd))
       exit(subprocess.call(cmd))
 
+    def compile_source_file(i, input_file):
+      logging.debug('compiling source file: ' + input_file)
+      output_file = get_bitcode_file(input_file)
+      temp_files.append((i, output_file))
+      args = get_bitcode_args([input_file]) + ['-emit-llvm', '-c', '-o', output_file]
+      logging.debug("running: " + ' '.join(args))
+      execute(args) # let compiler frontend print directly, so colors are saved (PIPE kills that)
+      if not os.path.exists(output_file):
+        logging.error('compiler frontend failed to generate LLVM bitcode, halting')
+        sys.exit(1)
+
     # First, generate LLVM bitcode. For each input file, we get base.o with bitcode
     for i, input_file in input_files:
       file_ending = filename_type_ending(input_file)
       if file_ending.endswith(SOURCE_ENDINGS):
-        logging.debug('compiling source file: ' + input_file)
-        output_file = get_bitcode_file(input_file)
-        temp_files.append((i, output_file))
-        args = get_bitcode_args([input_file]) + ['-emit-llvm', '-c', '-o', output_file]
-        logging.debug("running: " + ' '.join(args))
-        execute(args) # let compiler frontend print directly, so colors are saved (PIPE kills that)
-        if not os.path.exists(output_file):
-          logging.error('compiler frontend failed to generate LLVM bitcode, halting')
-          sys.exit(1)
+        compile_source_file(i, input_file)
       else: # bitcode
         if file_ending.endswith(BITCODE_ENDINGS):
           logging.debug('using bitcode file: ' + input_file)
@@ -1210,8 +1272,11 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
             shared.Building.llvm_as(input_file, temp_file)
             temp_files.append((i, temp_file))
         else:
-          logging.error(input_file + ': Unknown file suffix when compiling to LLVM bitcode!')
-          sys.exit(1)
+          if has_fixed_language_mode:
+            compile_source_file(i, input_file)
+          else:
+            logging.error(input_file + ': Unknown file suffix when compiling to LLVM bitcode!')
+            sys.exit(1)
 
     log_time('bitcodeize inputs')
 
@@ -1289,7 +1354,7 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
 
     log_time('calculate system libraries')
 
-    # final will be an array if linking is deferred, otherwise a normal string. 
+    # final will be an array if linking is deferred, otherwise a normal string.
     DEFAULT_FINAL = in_temp(target_basename + '.bc')
     def get_final():
       global final
@@ -1304,7 +1369,11 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       logging.debug('linking: ' + str(linker_inputs))
       # force archive contents to all be included, if just archives, or if linking shared modules
       force_archive_contents = len([temp for i, temp in temp_files if not temp.endswith(STATICLIB_ENDINGS)]) == 0 or not shared.Building.can_build_standalone()
-      just_calculate = DEBUG != '2' # if  EMCC_DEBUG=2 , link now, otherwise defer to be more efficient
+
+      # if  EMCC_DEBUG=2  then we must link now, so the temp files are complete.
+      # if using the wasm backend, we might be using vanilla LLVM, which does not allow our fastcomp deferred linking opts.
+      # TODO: we could check if this is a fastcomp build, and still speed things up here
+      just_calculate = DEBUG != '2' and not shared.Settings.WASM_BACKEND
       final = shared.Building.link(linker_inputs, DEFAULT_FINAL, force_archive_contents=force_archive_contents, temp_files=misc_temp_files, just_calculate=just_calculate)
     else:
       if not LEAVE_INPUTS_RAW:
@@ -1340,7 +1409,7 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
 
     # Optimize, if asked to
     if not LEAVE_INPUTS_RAW:
-      link_opts = [] if debug_level >= 4 else ['-strip-debug'] # remove LLVM debug if we are not asked for it
+      link_opts = [] if debug_level >= 4 or shared.Settings.CYBERDWARF else ['-strip-debug'] # remove LLVM debug if we are not asked for it
       if not shared.Settings.ASSERTIONS:
         link_opts += ['-disable-verify']
 
@@ -1403,10 +1472,14 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     if shared.Settings.WASM_BACKEND:
       # we also received wasm at this stage
       wasm_temp = final[:-3] + '.wast'
-      shutil.move(wasm_temp, wasm_target)
-      open(wasm_target + '.mappedGlobals', 'w').write('{}') # no need for mapped globals for now, but perhaps some day
+      shutil.move(wasm_temp, wasm_text_target)
+      open(wasm_text_target + '.mappedGlobals', 'w').write('{}') # no need for mapped globals for now, but perhaps some day
 
-    log_time('emscript (llvm=>%s)' % ('wasm' if shared.Settings.BINARYEN else 'js'))
+    if shared.Settings.CYBERDWARF:
+      cd_target = final + '.cd'
+      shutil.move(cd_target, target + '.cd')
+
+    log_time('emscript (llvm => executable code)')
 
     # Embed and preload files
     if shared.Settings.SPLIT_MEMORY:
@@ -1437,6 +1510,19 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       file_code = execute([shared.PYTHON, shared.FILE_PACKAGER, unsuffixed(target) + '.data'] + file_args, stdout=PIPE)[0]
       pre_js = file_code + pre_js
 
+    if shared.Settings.BINARYEN:
+      # add in the glue integration code as a pre-js, so it is optimized together with everything else
+      wasm_js_glue = open(os.path.join(shared.Settings.BINARYEN_ROOT, 'src', 'js', 'wasm.js-post.js')).read()
+      wasm_js_glue = wasm_js_glue.replace('{{{ asmjsCodeFile }}}', '"' + os.path.basename(asm_target) + '"')
+      wasm_js_glue = wasm_js_glue.replace('{{{ wasmTextFile }}}', '"' + os.path.basename(wasm_text_target) + '"')
+      wasm_js_glue = wasm_js_glue.replace('{{{ wasmBinaryFile }}}', '"' + os.path.basename(wasm_binary_target) + '"')
+      if shared.Settings.BINARYEN_METHOD:
+        wasm_js_glue = wasm_js_glue.replace('{{{ wasmJSMethod }}}', '(Module[\'wasmJSMethod\'] || "' + shared.Settings.BINARYEN_METHOD + '")')
+      else:
+        wasm_js_glue = wasm_js_glue.replace('{{{ wasmJSMethod }}}', 'null')
+      wasm_js_glue = wasm_js_glue.replace('{{{ WASM_BACKEND }}}', str(shared.Settings.WASM_BACKEND)) # if wasm backend, wasm contains memory segments
+      pre_js = wasm_js_glue + '\n' + pre_js
+
     # Apply pre and postjs files
     if pre_js or post_js:
       logging.debug('applying pre/postjses')
@@ -1445,7 +1531,12 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       if WINDOWS: # Avoid duplicating \r\n to \r\r\n when writing out.
         if pre_js: pre_js = pre_js.replace('\r\n', '\n')
         if post_js: post_js = post_js.replace('\r\n', '\n')
-      open(final, 'w').write(pre_js + src + post_js)
+      outfile = open(final, 'w')
+      outfile.write(pre_js)
+      outfile.write(src) # this may be large, don't join it to others
+      outfile.write(post_js)
+      outfile.close()
+      pre_js = src = post_js = None
       if DEBUG: save_intermediate('pre-post')
 
     # Apply a source code transformation, if requested
@@ -1538,7 +1629,7 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
             passes = ['asm'] + passes
             if shared.Settings.PRECISE_F32:
               passes = ['asmPreciseF32'] + passes
-            if emit_symbol_map and 'minifyNames' in passes:
+            if (emit_symbol_map or shared.Settings.CYBERDWARF) and 'minifyNames' in passes:
               passes += ['symbolMap='+target+'.symbols']
             if profiling_funcs and 'minifyNames' in passes:
               passes += ['profilingFuncs']
@@ -1761,6 +1852,10 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     #src = re.sub(r'\n+[ \n]*\n+', '\n', src)
     #open(final, 'w').write(src)
 
+    # Bundle symbol data in with the cyberdwarf file
+    if shared.Settings.CYBERDWARF:
+        execute([shared.PYTHON, shared.path_from_root('tools', 'emdebug_cd_merger.py'), target + '.cd', target+'.symbols'])
+
     # Emit source maps, if needed
     if debug_level >= 4:
       logging.debug('generating source maps')
@@ -1775,8 +1870,9 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
 
     # Separate out the asm.js code, if asked. Or, if necessary for another option
     if (separate_asm or shared.Settings.BINARYEN) and not shared.Settings.WASM_BACKEND:
+      logging.debug('separating asm')
       temp_target = misc_temp_files.get(suffix='.js').name
-      execute([shared.PYTHON, shared.path_from_root('tools', 'separate_asm.py'), js_target, asm_target, temp_target])
+      subprocess.check_call([shared.PYTHON, shared.path_from_root('tools', 'separate_asm.py'), js_target, asm_target, temp_target])
       shutil.move(temp_target, js_target)
 
       # extra only-my-code logic
@@ -1786,29 +1882,50 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
         shutil.move(temp, asm_target)
 
     if shared.Settings.BINARYEN:
-      # Emit wasm.js at the top of the js. TODO: for html, it could be a separate script tag
-      binaryen_bin = os.path.join(shared.BINARYEN_ROOT, 'bin')
-      wasm_js = open(os.path.join(binaryen_bin, 'wasm.js')).read()
-      wasm_js = wasm_js.replace("Module['asmjsCodeFile']", '"' + os.path.basename(asm_target) + '"') # " or '? who knows :)
-      wasm_js = wasm_js.replace('Module["asmjsCodeFile"]', '"' + os.path.basename(asm_target) + '"')
-      wasm_js = wasm_js.replace("Module['wasmCodeFile']", '"' + os.path.basename(wasm_target) + '"') # " or '? who knows :)
-      wasm_js = wasm_js.replace('Module["wasmCodeFile"]', '"' + os.path.basename(wasm_target) + '"')
-      wasm_js = wasm_js.replace("Module['providedTotalMemory']", str(shared.Settings.TOTAL_MEMORY))
-      wasm_js = wasm_js.replace('Module["providedTotalMemory"]', str(shared.Settings.TOTAL_MEMORY))
-      wasm_js = wasm_js.replace('EMSCRIPTEN_', 'emscripten_') # do not confuse the markers
-      if shared.Settings.BINARYEN_METHOD:
-        method = '(Module[\'wasmJSMethod\'] || "' + shared.Settings.BINARYEN_METHOD + '")'
-        wasm_js = wasm_js.replace("Module['wasmJSMethod']", method) # " or '? who knows :)
-        wasm_js = wasm_js.replace('Module["wasmJSMethod"]', method)
-      js = open(js_target).read()
-      combined = open(js_target, 'w')
-      combined.write(wasm_js)
-      combined.write('\n//^wasm.js\n')
-      combined.write(js)
-      combined.close()
+      logging.debug('using binaryen, with method: ' + shared.Settings.BINARYEN_METHOD)
+      binaryen_bin = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin')
+      # Emit wasm.js at the top of the js. This is *not* optimized with the rest of the code, since
+      # (1) it contains asm.js, whose validation would be broken, and (2) it's very large so it would
+      # be slow in cleanup/JSDCE etc.
+      # TODO: for html, it could be a separate script tag
+      # We need wasm.js if there is a chance the polyfill will be used. If the user sets
+      # BINARYEN_METHOD with something that doesn't use the polyfill, then we don't need it.
+      if not shared.Settings.BINARYEN_METHOD or 'interpret' in shared.Settings.BINARYEN_METHOD:
+        logging.debug('integrating wasm.js polyfill interpreter')
+        wasm_js = open(os.path.join(binaryen_bin, 'wasm.js')).read()
+        wasm_js = wasm_js.replace('EMSCRIPTEN_', 'emscripten_') # do not confuse the markers
+        js = open(js_target).read()
+        combined = open(js_target, 'w')
+        combined.write(wasm_js)
+        combined.write('\n//^wasm.js\n')
+        combined.write(js)
+        combined.close()
+      # finish compiling to WebAssembly, using asm2wasm, if we didn't already emit WebAssembly directly using the wasm backend.
       if not shared.Settings.WASM_BACKEND:
-        # generate .wast file
-        subprocess.check_call([os.path.join(binaryen_bin, 'asm2wasm'), asm_target, '--mapped-globals=' + wasm_target + '.mappedGlobals'], stdout=open(wasm_target, 'w'))
+        cmd = [os.path.join(binaryen_bin, 'asm2wasm'), asm_target, '--mapped-globals=' + wasm_text_target + '.mappedGlobals', '--total-memory=' + str(shared.Settings.TOTAL_MEMORY)]
+        if shared.Settings.BINARYEN_IMPRECISE:
+          cmd += ['--imprecise']
+        if opt_level == 0:
+          cmd += ['--no-opts']
+        logging.debug('asm2wasm (asm.js => WebAssembly): ' + ' '.join(cmd))
+        TimeLogger.update()
+        subprocess.check_call(cmd, stdout=open(wasm_text_target, 'w'))
+        log_time('asm2wasm')
+      if shared.Settings.BINARYEN_SCRIPTS:
+        binaryen_scripts = os.path.join(shared.Settings.BINARYEN_ROOT, 'scripts')
+        script_env = os.environ.copy()
+        root_dir = os.path.abspath(os.path.dirname(__file__))
+        if script_env.get('PYTHONPATH'):
+          script_env['PYTHONPATH'] += ':' + root_dir
+        else:
+          script_env['PYTHONPATH'] = root_dir
+        for script in shared.Settings.BINARYEN_SCRIPTS.split(','):
+          logging.debug('running binaryen script: ' + script)
+          subprocess.check_call([shared.PYTHON, os.path.join(binaryen_scripts, script), js_target, wasm_text_target], env=script_env)
+      if 'native-wasm' in shared.Settings.BINARYEN_METHOD or 'interpret-binary' in shared.Settings.BINARYEN_METHOD:
+        logging.debug('wasm-as (wasm => binary)')
+        subprocess.check_call([os.path.join(binaryen_bin, 'wasm-as'), wasm_text_target, '-o', wasm_binary_target])
+        shutil.copyfile(wasm_text_target + '.mappedGlobals', wasm_binary_target + '.mappedGlobals')
 
     # If we were asked to also generate HTML, do that
     if final_suffix == 'html':
@@ -1908,7 +2025,7 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       codeXHR = null;
       var src = URL.createObjectURL(blob);
       var script = document.createElement('script');
-      script.src = URL.createObjectURL(blob);
+      script.src = src;
       script.onload = function() {
         setTimeout(function() {
           %s
@@ -1922,6 +2039,20 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
       else:
         assert len(asm_mods) == 0, 'no --separate-asm means no client code mods are possible'
 
+      if shared.Settings.BINARYEN:
+        # We need to load the wasm file before anything else, it has to be synchronously ready TODO: optimize
+        un_src()
+        script_inline = '''
+          var xhr = new XMLHttpRequest();
+          xhr.open('GET', '%s', true);
+          xhr.responseType = 'arraybuffer';
+          xhr.onload = function() {
+            Module.wasmBinary = xhr.response;
+%s
+          };
+          xhr.send(null);
+''' % (os.path.basename(wasm_binary_target), script_inline)
+
       html = open(target, 'w')
       assert (script_src or script_inline) and not (script_src and script_inline)
       if script_src:
